@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 
-# Copyright (C) 2019  Google LLC
+# Copyright 2020 Google LLC
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -16,7 +16,9 @@
 #
 
 from collections import OrderedDict
-from typing import Dict, Sequence, Tuple, Type, Union
+import os
+import re
+from typing import Callable, Dict, Sequence, Tuple, Type, Union
 import pkg_resources
 
 import google.api_core.client_options as ClientOptions  # type: ignore
@@ -24,17 +26,22 @@ from google.api_core import exceptions  # type: ignore
 from google.api_core import gapic_v1  # type: ignore
 from google.api_core import retry as retries  # type: ignore
 from google.auth import credentials  # type: ignore
+from google.auth.transport import mtls  # type: ignore
+from google.auth.exceptions import MutualTLSChannelError  # type: ignore
 from google.oauth2 import service_account  # type: ignore
 
 from google.api_core import operation
+from google.api_core import operation_async
 from google.cloud.gaming_v1.services.game_server_clusters_service import pagers
 from google.cloud.gaming_v1.types import common
 from google.cloud.gaming_v1.types import game_server_clusters
+from google.protobuf import empty_pb2 as empty  # type: ignore
 from google.protobuf import field_mask_pb2 as field_mask  # type: ignore
 from google.protobuf import timestamp_pb2 as timestamp  # type: ignore
 
 from .transports.base import GameServerClustersServiceTransport
 from .transports.grpc import GameServerClustersServiceGrpcTransport
+from .transports.grpc_asyncio import GameServerClustersServiceGrpcAsyncIOTransport
 
 
 class GameServerClustersServiceClientMeta(type):
@@ -49,9 +56,10 @@ class GameServerClustersServiceClientMeta(type):
         OrderedDict()
     )  # type: Dict[str, Type[GameServerClustersServiceTransport]]
     _transport_registry["grpc"] = GameServerClustersServiceGrpcTransport
+    _transport_registry["grpc_asyncio"] = GameServerClustersServiceGrpcAsyncIOTransport
 
     def get_transport_class(
-        cls, label: str = None
+        cls, label: str = None,
     ) -> Type[GameServerClustersServiceTransport]:
         """Return an appropriate transport class.
 
@@ -76,8 +84,38 @@ class GameServerClustersServiceClient(metaclass=GameServerClustersServiceClientM
     Agones and is used to manage fleets within clusters.
     """
 
-    DEFAULT_OPTIONS = ClientOptions.ClientOptions(
-        api_endpoint="gameservices.googleapis.com"
+    @staticmethod
+    def _get_default_mtls_endpoint(api_endpoint):
+        """Convert api endpoint to mTLS endpoint.
+        Convert "*.sandbox.googleapis.com" and "*.googleapis.com" to
+        "*.mtls.sandbox.googleapis.com" and "*.mtls.googleapis.com" respectively.
+        Args:
+            api_endpoint (Optional[str]): the api endpoint to convert.
+        Returns:
+            str: converted mTLS api endpoint.
+        """
+        if not api_endpoint:
+            return api_endpoint
+
+        mtls_endpoint_re = re.compile(
+            r"(?P<name>[^.]+)(?P<mtls>\.mtls)?(?P<sandbox>\.sandbox)?(?P<googledomain>\.googleapis\.com)?"
+        )
+
+        m = mtls_endpoint_re.match(api_endpoint)
+        name, mtls, sandbox, googledomain = m.groups()
+        if mtls or not googledomain:
+            return api_endpoint
+
+        if sandbox:
+            return api_endpoint.replace(
+                "sandbox.googleapis.com", "mtls.sandbox.googleapis.com"
+            )
+
+        return api_endpoint.replace(".googleapis.com", ".mtls.googleapis.com")
+
+    DEFAULT_ENDPOINT = "gameservices.googleapis.com"
+    DEFAULT_MTLS_ENDPOINT = _get_default_mtls_endpoint.__func__(  # type: ignore
+        DEFAULT_ENDPOINT
     )
 
     @classmethod
@@ -102,19 +140,28 @@ class GameServerClustersServiceClient(metaclass=GameServerClustersServiceClientM
 
     @staticmethod
     def game_server_cluster_path(
-        project: str, location: str, realm: str, cluster: str
+        project: str, location: str, realm: str, cluster: str,
     ) -> str:
         """Return a fully-qualified game_server_cluster string."""
         return "projects/{project}/locations/{location}/realms/{realm}/gameServerClusters/{cluster}".format(
-            project=project, location=location, realm=realm, cluster=cluster
+            project=project, location=location, realm=realm, cluster=cluster,
         )
 
+    @staticmethod
+    def parse_game_server_cluster_path(path: str) -> Dict[str, str]:
+        """Parse a game_server_cluster path into its component segments."""
+        m = re.match(
+            r"^projects/(?P<project>.+?)/locations/(?P<location>.+?)/realms/(?P<realm>.+?)/gameServerClusters/(?P<cluster>.+?)$",
+            path,
+        )
+        return m.groupdict() if m else {}
+
     def __init__(
         self,
         *,
         credentials: credentials.Credentials = None,
         transport: Union[str, GameServerClustersServiceTransport] = None,
-        client_options: ClientOptions = DEFAULT_OPTIONS,
+        client_options: ClientOptions = None,
     ) -> None:
         """Instantiate the game server clusters service client.
 
@@ -127,26 +174,75 @@ class GameServerClustersServiceClient(metaclass=GameServerClustersServiceClientM
             transport (Union[str, ~.GameServerClustersServiceTransport]): The
                 transport to use. If set to None, a transport is chosen
                 automatically.
-            client_options (ClientOptions): Custom options for the client.
+            client_options (ClientOptions): Custom options for the client. It
+                won't take effect if a ``transport`` instance is provided.
+                (1) The ``api_endpoint`` property can be used to override the
+                default endpoint provided by the client. GOOGLE_API_USE_MTLS
+                environment variable can also be used to override the endpoint:
+                "always" (always use the default mTLS endpoint), "never" (always
+                use the default regular endpoint, this is the default value for
+                the environment variable) and "auto" (auto switch to the default
+                mTLS endpoint if client SSL credentials is present). However,
+                the ``api_endpoint`` property takes precedence if provided.
+                (2) The ``client_cert_source`` property is used to provide client
+                SSL credentials for mutual TLS transport. If not provided, the
+                default SSL credentials will be used if present.
+
+        Raises:
+            google.auth.exceptions.MutualTLSChannelError: If mutual TLS transport
+                creation failed for any reason.
         """
         if isinstance(client_options, dict):
             client_options = ClientOptions.from_dict(client_options)
+        if client_options is None:
+            client_options = ClientOptions.ClientOptions()
+
+        if client_options.api_endpoint is None:
+            use_mtls_env = os.getenv("GOOGLE_API_USE_MTLS", "never")
+            if use_mtls_env == "never":
+                client_options.api_endpoint = self.DEFAULT_ENDPOINT
+            elif use_mtls_env == "always":
+                client_options.api_endpoint = self.DEFAULT_MTLS_ENDPOINT
+            elif use_mtls_env == "auto":
+                has_client_cert_source = (
+                    client_options.client_cert_source is not None
+                    or mtls.has_default_client_cert_source()
+                )
+                client_options.api_endpoint = (
+                    self.DEFAULT_MTLS_ENDPOINT
+                    if has_client_cert_source
+                    else self.DEFAULT_ENDPOINT
+                )
+            else:
+                raise MutualTLSChannelError(
+                    "Unsupported GOOGLE_API_USE_MTLS value. Accepted values: never, auto, always"
+                )
 
         # Save or instantiate the transport.
         # Ordinarily, we provide the transport, but allowing a custom transport
         # instance provides an extensibility point for unusual situations.
         if isinstance(transport, GameServerClustersServiceTransport):
-            if credentials:
+            # transport is a GameServerClustersServiceTransport instance.
+            if credentials or client_options.credentials_file:
                 raise ValueError(
                     "When providing a transport instance, "
                     "provide its credentials directly."
                 )
+            if client_options.scopes:
+                raise ValueError(
+                    "When providing a transport instance, "
+                    "provide its scopes directly."
+                )
             self._transport = transport
         else:
             Transport = type(self).get_transport_class(transport)
             self._transport = Transport(
                 credentials=credentials,
-                host=client_options.api_endpoint or "gameservices.googleapis.com",
+                credentials_file=client_options.credentials_file,
+                host=client_options.api_endpoint,
+                scopes=client_options.scopes,
+                api_mtls_endpoint=client_options.api_endpoint,
+                client_cert_source=client_options.client_cert_source,
             )
 
     def list_game_server_clusters(
@@ -158,7 +254,7 @@ class GameServerClustersServiceClient(metaclass=GameServerClustersServiceClientM
         timeout: float = None,
         metadata: Sequence[Tuple[str, str]] = (),
     ) -> pagers.ListGameServerClustersPager:
-        r"""Lists Game Server Clusters in a given project and
+        r"""Lists game server clusters in a given project and
         location.
 
         Args:
@@ -201,6 +297,7 @@ class GameServerClustersServiceClient(metaclass=GameServerClustersServiceClientM
 
         # If we have keyword arguments corresponding to fields on the
         # request, apply these.
+
         if parent is not None:
             request.parent = parent
 
@@ -219,12 +316,12 @@ class GameServerClustersServiceClient(metaclass=GameServerClustersServiceClientM
         )
 
         # Send the request.
-        response = rpc(request, retry=retry, timeout=timeout, metadata=metadata)
+        response = rpc(request, retry=retry, timeout=timeout, metadata=metadata,)
 
         # This method is paged; wrap the response in a pager, which provides
         # an `__iter__` convenience method.
         response = pagers.ListGameServerClustersPager(
-            method=rpc, request=request, response=response
+            method=rpc, request=request, response=response, metadata=metadata,
         )
 
         # Done; return the response.
@@ -246,7 +343,7 @@ class GameServerClustersServiceClient(metaclass=GameServerClustersServiceClientM
                 The request object. Request message for
                 GameServerClustersService.GetGameServerCluster.
             name (:class:`str`):
-                Required. The name of the Game Server Cluster to
+                Required. The name of the game server cluster to
                 retrieve. Uses the form:
 
                 ``projects/{project}/locations/{location}/realms/{realm-id}/gameServerClusters/{cluster}``.
@@ -262,7 +359,7 @@ class GameServerClustersServiceClient(metaclass=GameServerClustersServiceClientM
 
         Returns:
             ~.game_server_clusters.GameServerCluster:
-                A Game Server Cluster resource.
+                A game server cluster resource.
         """
         # Create or coerce a protobuf request object.
         # Sanity check: If we got a request object, we should *not* have
@@ -277,6 +374,7 @@ class GameServerClustersServiceClient(metaclass=GameServerClustersServiceClientM
 
         # If we have keyword arguments corresponding to fields on the
         # request, apply these.
+
         if name is not None:
             request.name = name
 
@@ -295,7 +393,7 @@ class GameServerClustersServiceClient(metaclass=GameServerClustersServiceClientM
         )
 
         # Send the request.
-        response = rpc(request, retry=retry, timeout=timeout, metadata=metadata)
+        response = rpc(request, retry=retry, timeout=timeout, metadata=metadata,)
 
         # Done; return the response.
         return response
@@ -325,14 +423,14 @@ class GameServerClustersServiceClient(metaclass=GameServerClustersServiceClientM
                 on the ``request`` instance; if ``request`` is provided, this
                 should not be set.
             game_server_cluster (:class:`~.game_server_clusters.GameServerCluster`):
-                Required. The Game Server Cluster
+                Required. The game server cluster
                 resource to be created.
                 This corresponds to the ``game_server_cluster`` field
                 on the ``request`` instance; if ``request`` is provided, this
                 should not be set.
             game_server_cluster_id (:class:`str`):
-                Required. The ID of the Game Server
-                Cluster resource to be created.
+                Required. The ID of the game server
+                cluster resource to be created.
                 This corresponds to the ``game_server_cluster_id`` field
                 on the ``request`` instance; if ``request`` is provided, this
                 should not be set.
@@ -349,7 +447,7 @@ class GameServerClustersServiceClient(metaclass=GameServerClustersServiceClientM
 
                 The result type for the operation will be
                 :class:``~.game_server_clusters.GameServerCluster``: A
-                Game Server Cluster resource.
+                game server cluster resource.
 
         """
         # Create or coerce a protobuf request object.
@@ -367,14 +465,11 @@ class GameServerClustersServiceClient(metaclass=GameServerClustersServiceClientM
 
         # If we have keyword arguments corresponding to fields on the
         # request, apply these.
+
         if parent is not None:
             request.parent = parent
-        # If we have keyword arguments corresponding to fields on the
-        # request, apply these.
         if game_server_cluster is not None:
             request.game_server_cluster = game_server_cluster
-        # If we have keyword arguments corresponding to fields on the
-        # request, apply these.
         if game_server_cluster_id is not None:
             request.game_server_cluster_id = game_server_cluster_id
 
@@ -386,8 +481,14 @@ class GameServerClustersServiceClient(metaclass=GameServerClustersServiceClientM
             client_info=_client_info,
         )
 
+        # Certain fields should be provided within the metadata header;
+        # add these here.
+        metadata = tuple(metadata) + (
+            gapic_v1.routing_header.to_grpc_metadata((("parent", request.parent),)),
+        )
+
         # Send the request.
-        response = rpc(request, retry=retry, timeout=timeout, metadata=metadata)
+        response = rpc(request, retry=retry, timeout=timeout, metadata=metadata,)
 
         # Wrap the response in an operation future.
         response = operation.from_gapic(
@@ -440,8 +541,14 @@ class GameServerClustersServiceClient(metaclass=GameServerClustersServiceClientM
             client_info=_client_info,
         )
 
+        # Certain fields should be provided within the metadata header;
+        # add these here.
+        metadata = tuple(metadata) + (
+            gapic_v1.routing_header.to_grpc_metadata((("parent", request.parent),)),
+        )
+
         # Send the request.
-        response = rpc(request, retry=retry, timeout=timeout, metadata=metadata)
+        response = rpc(request, retry=retry, timeout=timeout, metadata=metadata,)
 
         # Done; return the response.
         return response
@@ -462,7 +569,7 @@ class GameServerClustersServiceClient(metaclass=GameServerClustersServiceClientM
                 The request object. Request message for
                 GameServerClustersService.DeleteGameServerCluster.
             name (:class:`str`):
-                Required. The name of the Game Server Cluster to delete.
+                Required. The name of the game server cluster to delete.
                 Uses the form:
                 ``projects/{project}/locations/{location}/gameServerClusters/{cluster}``.
                 This corresponds to the ``name`` field
@@ -480,8 +587,20 @@ class GameServerClustersServiceClient(metaclass=GameServerClustersServiceClientM
                 An object representing a long-running operation.
 
                 The result type for the operation will be
-                :class:``~.game_server_clusters.GameServerCluster``: A
-                Game Server Cluster resource.
+                :class:``~.empty.Empty``: A generic empty message that
+                you can re-use to avoid defining duplicated empty
+                messages in your APIs. A typical example is to use it as
+                the request or the response type of an API method. For
+                instance:
+
+                ::
+
+                    service Foo {
+                      rpc Bar(google.protobuf.Empty) returns (google.protobuf.Empty);
+                    }
+
+                The JSON representation for ``Empty`` is empty JSON
+                object ``{}``.
 
         """
         # Create or coerce a protobuf request object.
@@ -497,6 +616,7 @@ class GameServerClustersServiceClient(metaclass=GameServerClustersServiceClientM
 
         # If we have keyword arguments corresponding to fields on the
         # request, apply these.
+
         if name is not None:
             request.name = name
 
@@ -508,14 +628,20 @@ class GameServerClustersServiceClient(metaclass=GameServerClustersServiceClientM
             client_info=_client_info,
         )
 
+        # Certain fields should be provided within the metadata header;
+        # add these here.
+        metadata = tuple(metadata) + (
+            gapic_v1.routing_header.to_grpc_metadata((("name", request.name),)),
+        )
+
         # Send the request.
-        response = rpc(request, retry=retry, timeout=timeout, metadata=metadata)
+        response = rpc(request, retry=retry, timeout=timeout, metadata=metadata,)
 
         # Wrap the response in an operation future.
         response = operation.from_gapic(
             response,
             self._transport.operations_client,
-            game_server_clusters.GameServerCluster,
+            empty.Empty,
             metadata_type=common.OperationMetadata,
         )
 
@@ -561,8 +687,14 @@ class GameServerClustersServiceClient(metaclass=GameServerClustersServiceClientM
             client_info=_client_info,
         )
 
+        # Certain fields should be provided within the metadata header;
+        # add these here.
+        metadata = tuple(metadata) + (
+            gapic_v1.routing_header.to_grpc_metadata((("name", request.name),)),
+        )
+
         # Send the request.
-        response = rpc(request, retry=retry, timeout=timeout, metadata=metadata)
+        response = rpc(request, retry=retry, timeout=timeout, metadata=metadata,)
 
         # Done; return the response.
         return response
@@ -584,7 +716,7 @@ class GameServerClustersServiceClient(metaclass=GameServerClustersServiceClientM
                 The request object. Request message for
                 GameServerClustersService.UpdateGameServerCluster.
             game_server_cluster (:class:`~.game_server_clusters.GameServerCluster`):
-                Required. The Game Server Cluster to be updated. Only
+                Required. The game server cluster to be updated. Only
                 fields specified in update_mask are updated.
                 This corresponds to the ``game_server_cluster`` field
                 on the ``request`` instance; if ``request`` is provided, this
@@ -612,7 +744,7 @@ class GameServerClustersServiceClient(metaclass=GameServerClustersServiceClientM
 
                 The result type for the operation will be
                 :class:``~.game_server_clusters.GameServerCluster``: A
-                Game Server Cluster resource.
+                game server cluster resource.
 
         """
         # Create or coerce a protobuf request object.
@@ -628,10 +760,9 @@ class GameServerClustersServiceClient(metaclass=GameServerClustersServiceClientM
 
         # If we have keyword arguments corresponding to fields on the
         # request, apply these.
+
         if game_server_cluster is not None:
             request.game_server_cluster = game_server_cluster
-        # If we have keyword arguments corresponding to fields on the
-        # request, apply these.
         if update_mask is not None:
             request.update_mask = update_mask
 
@@ -643,8 +774,16 @@ class GameServerClustersServiceClient(metaclass=GameServerClustersServiceClientM
             client_info=_client_info,
         )
 
+        # Certain fields should be provided within the metadata header;
+        # add these here.
+        metadata = tuple(metadata) + (
+            gapic_v1.routing_header.to_grpc_metadata(
+                (("game_server_cluster.name", request.game_server_cluster.name),)
+            ),
+        )
+
         # Send the request.
-        response = rpc(request, retry=retry, timeout=timeout, metadata=metadata)
+        response = rpc(request, retry=retry, timeout=timeout, metadata=metadata,)
 
         # Wrap the response in an operation future.
         response = operation.from_gapic(
@@ -696,8 +835,16 @@ class GameServerClustersServiceClient(metaclass=GameServerClustersServiceClientM
             client_info=_client_info,
         )
 
+        # Certain fields should be provided within the metadata header;
+        # add these here.
+        metadata = tuple(metadata) + (
+            gapic_v1.routing_header.to_grpc_metadata(
+                (("game_server_cluster.name", request.game_server_cluster.name),)
+            ),
+        )
+
         # Send the request.
-        response = rpc(request, retry=retry, timeout=timeout, metadata=metadata)
+        response = rpc(request, retry=retry, timeout=timeout, metadata=metadata,)
 
         # Done; return the response.
         return response
@@ -706,8 +853,8 @@ class GameServerClustersServiceClient(metaclass=GameServerClustersServiceClientM
 try:
     _client_info = gapic_v1.client_info.ClientInfo(
         gapic_version=pkg_resources.get_distribution(
-            "google-cloud-game-servers"
-        ).version
+            "google-cloud-game-servers",
+        ).version,
     )
 except pkg_resources.DistributionNotFound:
     _client_info = gapic_v1.client_info.ClientInfo()
