@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 
-# Copyright (C) 2019  Google LLC
+# Copyright 2020 Google LLC
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -15,11 +15,14 @@
 # limitations under the License.
 #
 
-from typing import Callable, Dict
+from typing import Callable, Dict, Optional, Sequence, Tuple
 
 from google.api_core import grpc_helpers  # type: ignore
 from google.api_core import operations_v1  # type: ignore
+from google import auth  # type: ignore
 from google.auth import credentials  # type: ignore
+from google.auth.transport.grpc import SslCredentials  # type: ignore
+
 
 import grpc  # type: ignore
 
@@ -43,12 +46,18 @@ class GameServerClustersServiceGrpcTransport(GameServerClustersServiceTransport)
     top of HTTP/2); the ``grpcio`` package must be installed.
     """
 
+    _stubs: Dict[str, Callable]
+
     def __init__(
         self,
         *,
         host: str = "gameservices.googleapis.com",
         credentials: credentials.Credentials = None,
-        channel: grpc.Channel = None
+        credentials_file: str = None,
+        scopes: Sequence[str] = None,
+        channel: grpc.Channel = None,
+        api_mtls_endpoint: str = None,
+        client_cert_source: Callable[[], Tuple[bytes, bytes]] = None
     ) -> None:
         """Instantiate the transport.
 
@@ -60,27 +69,81 @@ class GameServerClustersServiceGrpcTransport(GameServerClustersServiceTransport)
                 are specified, the client will attempt to ascertain the
                 credentials from the environment.
                 This argument is ignored if ``channel`` is provided.
+            credentials_file (Optional[str]): A file with credentials that can
+                be loaded with :func:`google.auth.load_credentials_from_file`.
+                This argument is ignored if ``channel`` is provided.
+            scopes (Optional(Sequence[str])): A list of scopes. This argument is
+                ignored if ``channel`` is provided.
             channel (Optional[grpc.Channel]): A ``Channel`` instance through
                 which to make calls.
+            api_mtls_endpoint (Optional[str]): The mutual TLS endpoint. If
+                provided, it overrides the ``host`` argument and tries to create
+                a mutual TLS channel with client SSL credentials from
+                ``client_cert_source`` or applicatin default SSL credentials.
+            client_cert_source (Optional[Callable[[], Tuple[bytes, bytes]]]): A
+                callback to provide client SSL certificate bytes and private key
+                bytes, both in PEM format. It is ignored if ``api_mtls_endpoint``
+                is None.
+
+        Raises:
+          google.auth.exceptions.MutualTLSChannelError: If mutual TLS transport
+              creation failed for any reason.
+          google.api_core.exceptions.DuplicateCredentialArgs: If both ``credentials``
+              and ``credentials_file`` are passed.
         """
-        # Sanity check: Ensure that channel and credentials are not both
-        # provided.
         if channel:
+            # Sanity check: Ensure that channel and credentials are not both
+            # provided.
             credentials = False
 
-        # Run the base constructor.
-        super().__init__(host=host, credentials=credentials)
-        self._stubs = {}  # type: Dict[str, Callable]
-
-        # If a channel was explicitly provided, set it.
-        if channel:
+            # If a channel was explicitly provided, set it.
             self._grpc_channel = channel
+        elif api_mtls_endpoint:
+            host = (
+                api_mtls_endpoint
+                if ":" in api_mtls_endpoint
+                else api_mtls_endpoint + ":443"
+            )
+
+            if credentials is None:
+                credentials, _ = auth.default(scopes=self.AUTH_SCOPES)
+
+            # Create SSL credentials with client_cert_source or application
+            # default SSL credentials.
+            if client_cert_source:
+                cert, key = client_cert_source()
+                ssl_credentials = grpc.ssl_channel_credentials(
+                    certificate_chain=cert, private_key=key
+                )
+            else:
+                ssl_credentials = SslCredentials().ssl_credentials
+
+            # create a new channel. The provided one is ignored.
+            self._grpc_channel = type(self).create_channel(
+                host,
+                credentials=credentials,
+                credentials_file=credentials_file,
+                ssl_credentials=ssl_credentials,
+                scopes=scopes or self.AUTH_SCOPES,
+            )
+
+        # Run the base constructor.
+        super().__init__(
+            host=host,
+            credentials=credentials,
+            credentials_file=credentials_file,
+            scopes=scopes or self.AUTH_SCOPES,
+        )
+
+        self._stubs = {}  # type: Dict[str, Callable]
 
     @classmethod
     def create_channel(
         cls,
         host: str = "gameservices.googleapis.com",
         credentials: credentials.Credentials = None,
+        credentials_file: str = None,
+        scopes: Optional[Sequence[str]] = None,
         **kwargs
     ) -> grpc.Channel:
         """Create and return a gRPC channel object.
@@ -91,13 +154,28 @@ class GameServerClustersServiceGrpcTransport(GameServerClustersServiceTransport)
                 credentials identify this application to the service. If
                 none are specified, the client will attempt to ascertain
                 the credentials from the environment.
+            credentials_file (Optional[str]): A file with credentials that can
+                be loaded with :func:`google.auth.load_credentials_from_file`.
+                This argument is mutually exclusive with credentials.
+            scopes (Optional[Sequence[str]]): A optional list of scopes needed for this
+                service. These are only used when credentials are not specified and
+                are passed to :func:`google.auth.default`.
             kwargs (Optional[dict]): Keyword arguments, which are passed to the
                 channel creation.
         Returns:
             grpc.Channel: A gRPC channel object.
+
+        Raises:
+            google.api_core.exceptions.DuplicateCredentialArgs: If both ``credentials``
+              and ``credentials_file`` are passed.
         """
+        scopes = scopes or cls.AUTH_SCOPES
         return grpc_helpers.create_channel(
-            host, credentials=credentials, scopes=cls.AUTH_SCOPES, **kwargs
+            host,
+            credentials=credentials,
+            credentials_file=credentials_file,
+            scopes=scopes,
+            **kwargs
         )
 
     @property
@@ -111,7 +189,7 @@ class GameServerClustersServiceGrpcTransport(GameServerClustersServiceTransport)
         # have one.
         if not hasattr(self, "_grpc_channel"):
             self._grpc_channel = self.create_channel(
-                self._host, credentials=self._credentials
+                self._host, credentials=self._credentials,
             )
 
         # Return the channel from cache.
@@ -135,14 +213,14 @@ class GameServerClustersServiceGrpcTransport(GameServerClustersServiceTransport)
 
     @property
     def list_game_server_clusters(
-        self
+        self,
     ) -> Callable[
         [game_server_clusters.ListGameServerClustersRequest],
         game_server_clusters.ListGameServerClustersResponse,
     ]:
         r"""Return a callable for the list game server clusters method over gRPC.
 
-        Lists Game Server Clusters in a given project and
+        Lists game server clusters in a given project and
         location.
 
         Returns:
@@ -165,7 +243,7 @@ class GameServerClustersServiceGrpcTransport(GameServerClustersServiceTransport)
 
     @property
     def get_game_server_cluster(
-        self
+        self,
     ) -> Callable[
         [game_server_clusters.GetGameServerClusterRequest],
         game_server_clusters.GameServerCluster,
@@ -194,7 +272,7 @@ class GameServerClustersServiceGrpcTransport(GameServerClustersServiceTransport)
 
     @property
     def create_game_server_cluster(
-        self
+        self,
     ) -> Callable[
         [game_server_clusters.CreateGameServerClusterRequest], operations.Operation
     ]:
@@ -223,7 +301,7 @@ class GameServerClustersServiceGrpcTransport(GameServerClustersServiceTransport)
 
     @property
     def preview_create_game_server_cluster(
-        self
+        self,
     ) -> Callable[
         [game_server_clusters.PreviewCreateGameServerClusterRequest],
         game_server_clusters.PreviewCreateGameServerClusterResponse,
@@ -256,7 +334,7 @@ class GameServerClustersServiceGrpcTransport(GameServerClustersServiceTransport)
 
     @property
     def delete_game_server_cluster(
-        self
+        self,
     ) -> Callable[
         [game_server_clusters.DeleteGameServerClusterRequest], operations.Operation
     ]:
@@ -284,7 +362,7 @@ class GameServerClustersServiceGrpcTransport(GameServerClustersServiceTransport)
 
     @property
     def preview_delete_game_server_cluster(
-        self
+        self,
     ) -> Callable[
         [game_server_clusters.PreviewDeleteGameServerClusterRequest],
         game_server_clusters.PreviewDeleteGameServerClusterResponse,
@@ -316,7 +394,7 @@ class GameServerClustersServiceGrpcTransport(GameServerClustersServiceTransport)
 
     @property
     def update_game_server_cluster(
-        self
+        self,
     ) -> Callable[
         [game_server_clusters.UpdateGameServerClusterRequest], operations.Operation
     ]:
@@ -344,7 +422,7 @@ class GameServerClustersServiceGrpcTransport(GameServerClustersServiceTransport)
 
     @property
     def preview_update_game_server_cluster(
-        self
+        self,
     ) -> Callable[
         [game_server_clusters.PreviewUpdateGameServerClusterRequest],
         game_server_clusters.PreviewUpdateGameServerClusterResponse,
