@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 
-# Copyright (C) 2019  Google LLC
+# Copyright 2020 Google LLC
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -16,7 +16,9 @@
 #
 
 from collections import OrderedDict
-from typing import Dict, Sequence, Tuple, Type, Union
+import os
+import re
+from typing import Callable, Dict, Sequence, Tuple, Type, Union
 import pkg_resources
 
 import google.api_core.client_options as ClientOptions  # type: ignore
@@ -24,16 +26,21 @@ from google.api_core import exceptions  # type: ignore
 from google.api_core import gapic_v1  # type: ignore
 from google.api_core import retry as retries  # type: ignore
 from google.auth import credentials  # type: ignore
+from google.auth.transport import mtls  # type: ignore
+from google.auth.exceptions import MutualTLSChannelError  # type: ignore
 from google.oauth2 import service_account  # type: ignore
 
 from google.api_core import operation
+from google.api_core import operation_async
 from google.cloud.gaming_v1.services.game_server_configs_service import pagers
 from google.cloud.gaming_v1.types import common
 from google.cloud.gaming_v1.types import game_server_configs
+from google.protobuf import empty_pb2 as empty  # type: ignore
 from google.protobuf import timestamp_pb2 as timestamp  # type: ignore
 
 from .transports.base import GameServerConfigsServiceTransport
 from .transports.grpc import GameServerConfigsServiceGrpcTransport
+from .transports.grpc_asyncio import GameServerConfigsServiceGrpcAsyncIOTransport
 
 
 class GameServerConfigsServiceClientMeta(type):
@@ -48,9 +55,10 @@ class GameServerConfigsServiceClientMeta(type):
         OrderedDict()
     )  # type: Dict[str, Type[GameServerConfigsServiceTransport]]
     _transport_registry["grpc"] = GameServerConfigsServiceGrpcTransport
+    _transport_registry["grpc_asyncio"] = GameServerConfigsServiceGrpcAsyncIOTransport
 
     def get_transport_class(
-        cls, label: str = None
+        cls, label: str = None,
     ) -> Type[GameServerConfigsServiceTransport]:
         """Return an appropriate transport class.
 
@@ -71,12 +79,42 @@ class GameServerConfigsServiceClientMeta(type):
 
 
 class GameServerConfigsServiceClient(metaclass=GameServerConfigsServiceClientMeta):
-    """The Game Server Config configures the game servers in an
+    """The game server config configures the game servers in an
     Agones fleet.
     """
 
-    DEFAULT_OPTIONS = ClientOptions.ClientOptions(
-        api_endpoint="gameservices.googleapis.com"
+    @staticmethod
+    def _get_default_mtls_endpoint(api_endpoint):
+        """Convert api endpoint to mTLS endpoint.
+        Convert "*.sandbox.googleapis.com" and "*.googleapis.com" to
+        "*.mtls.sandbox.googleapis.com" and "*.mtls.googleapis.com" respectively.
+        Args:
+            api_endpoint (Optional[str]): the api endpoint to convert.
+        Returns:
+            str: converted mTLS api endpoint.
+        """
+        if not api_endpoint:
+            return api_endpoint
+
+        mtls_endpoint_re = re.compile(
+            r"(?P<name>[^.]+)(?P<mtls>\.mtls)?(?P<sandbox>\.sandbox)?(?P<googledomain>\.googleapis\.com)?"
+        )
+
+        m = mtls_endpoint_re.match(api_endpoint)
+        name, mtls, sandbox, googledomain = m.groups()
+        if mtls or not googledomain:
+            return api_endpoint
+
+        if sandbox:
+            return api_endpoint.replace(
+                "sandbox.googleapis.com", "mtls.sandbox.googleapis.com"
+            )
+
+        return api_endpoint.replace(".googleapis.com", ".mtls.googleapis.com")
+
+    DEFAULT_ENDPOINT = "gameservices.googleapis.com"
+    DEFAULT_MTLS_ENDPOINT = _get_default_mtls_endpoint.__func__(  # type: ignore
+        DEFAULT_ENDPOINT
     )
 
     @classmethod
@@ -101,19 +139,28 @@ class GameServerConfigsServiceClient(metaclass=GameServerConfigsServiceClientMet
 
     @staticmethod
     def game_server_config_path(
-        project: str, location: str, deployment: str, config: str
+        project: str, location: str, deployment: str, config: str,
     ) -> str:
         """Return a fully-qualified game_server_config string."""
         return "projects/{project}/locations/{location}/gameServerDeployments/{deployment}/configs/{config}".format(
-            project=project, location=location, deployment=deployment, config=config
+            project=project, location=location, deployment=deployment, config=config,
         )
 
+    @staticmethod
+    def parse_game_server_config_path(path: str) -> Dict[str, str]:
+        """Parse a game_server_config path into its component segments."""
+        m = re.match(
+            r"^projects/(?P<project>.+?)/locations/(?P<location>.+?)/gameServerDeployments/(?P<deployment>.+?)/configs/(?P<config>.+?)$",
+            path,
+        )
+        return m.groupdict() if m else {}
+
     def __init__(
         self,
         *,
         credentials: credentials.Credentials = None,
         transport: Union[str, GameServerConfigsServiceTransport] = None,
-        client_options: ClientOptions = DEFAULT_OPTIONS,
+        client_options: ClientOptions = None,
     ) -> None:
         """Instantiate the game server configs service client.
 
@@ -126,26 +173,75 @@ class GameServerConfigsServiceClient(metaclass=GameServerConfigsServiceClientMet
             transport (Union[str, ~.GameServerConfigsServiceTransport]): The
                 transport to use. If set to None, a transport is chosen
                 automatically.
-            client_options (ClientOptions): Custom options for the client.
+            client_options (ClientOptions): Custom options for the client. It
+                won't take effect if a ``transport`` instance is provided.
+                (1) The ``api_endpoint`` property can be used to override the
+                default endpoint provided by the client. GOOGLE_API_USE_MTLS
+                environment variable can also be used to override the endpoint:
+                "always" (always use the default mTLS endpoint), "never" (always
+                use the default regular endpoint, this is the default value for
+                the environment variable) and "auto" (auto switch to the default
+                mTLS endpoint if client SSL credentials is present). However,
+                the ``api_endpoint`` property takes precedence if provided.
+                (2) The ``client_cert_source`` property is used to provide client
+                SSL credentials for mutual TLS transport. If not provided, the
+                default SSL credentials will be used if present.
+
+        Raises:
+            google.auth.exceptions.MutualTLSChannelError: If mutual TLS transport
+                creation failed for any reason.
         """
         if isinstance(client_options, dict):
             client_options = ClientOptions.from_dict(client_options)
+        if client_options is None:
+            client_options = ClientOptions.ClientOptions()
+
+        if client_options.api_endpoint is None:
+            use_mtls_env = os.getenv("GOOGLE_API_USE_MTLS", "never")
+            if use_mtls_env == "never":
+                client_options.api_endpoint = self.DEFAULT_ENDPOINT
+            elif use_mtls_env == "always":
+                client_options.api_endpoint = self.DEFAULT_MTLS_ENDPOINT
+            elif use_mtls_env == "auto":
+                has_client_cert_source = (
+                    client_options.client_cert_source is not None
+                    or mtls.has_default_client_cert_source()
+                )
+                client_options.api_endpoint = (
+                    self.DEFAULT_MTLS_ENDPOINT
+                    if has_client_cert_source
+                    else self.DEFAULT_ENDPOINT
+                )
+            else:
+                raise MutualTLSChannelError(
+                    "Unsupported GOOGLE_API_USE_MTLS value. Accepted values: never, auto, always"
+                )
 
         # Save or instantiate the transport.
         # Ordinarily, we provide the transport, but allowing a custom transport
         # instance provides an extensibility point for unusual situations.
         if isinstance(transport, GameServerConfigsServiceTransport):
-            if credentials:
+            # transport is a GameServerConfigsServiceTransport instance.
+            if credentials or client_options.credentials_file:
                 raise ValueError(
                     "When providing a transport instance, "
                     "provide its credentials directly."
                 )
+            if client_options.scopes:
+                raise ValueError(
+                    "When providing a transport instance, "
+                    "provide its scopes directly."
+                )
             self._transport = transport
         else:
             Transport = type(self).get_transport_class(transport)
             self._transport = Transport(
                 credentials=credentials,
-                host=client_options.api_endpoint or "gameservices.googleapis.com",
+                credentials_file=client_options.credentials_file,
+                host=client_options.api_endpoint,
+                scopes=client_options.scopes,
+                api_mtls_endpoint=client_options.api_endpoint,
+                client_cert_source=client_options.client_cert_source,
             )
 
     def list_game_server_configs(
@@ -157,8 +253,8 @@ class GameServerConfigsServiceClient(metaclass=GameServerConfigsServiceClientMet
         timeout: float = None,
         metadata: Sequence[Tuple[str, str]] = (),
     ) -> pagers.ListGameServerConfigsPager:
-        r"""Lists Game Server Configs in a given project,
-        Location, and Game Server Deployment.
+        r"""Lists game server configs in a given project,
+        location, and game server deployment.
 
         Args:
             request (:class:`~.game_server_configs.ListGameServerConfigsRequest`):
@@ -200,6 +296,7 @@ class GameServerConfigsServiceClient(metaclass=GameServerConfigsServiceClientMet
 
         # If we have keyword arguments corresponding to fields on the
         # request, apply these.
+
         if parent is not None:
             request.parent = parent
 
@@ -218,12 +315,12 @@ class GameServerConfigsServiceClient(metaclass=GameServerConfigsServiceClientMet
         )
 
         # Send the request.
-        response = rpc(request, retry=retry, timeout=timeout, metadata=metadata)
+        response = rpc(request, retry=retry, timeout=timeout, metadata=metadata,)
 
         # This method is paged; wrap the response in a pager, which provides
         # an `__iter__` convenience method.
         response = pagers.ListGameServerConfigsPager(
-            method=rpc, request=request, response=response
+            method=rpc, request=request, response=response, metadata=metadata,
         )
 
         # Done; return the response.
@@ -238,14 +335,14 @@ class GameServerConfigsServiceClient(metaclass=GameServerConfigsServiceClientMet
         timeout: float = None,
         metadata: Sequence[Tuple[str, str]] = (),
     ) -> game_server_configs.GameServerConfig:
-        r"""Gets details of a single Game Server Config.
+        r"""Gets details of a single game server config.
 
         Args:
             request (:class:`~.game_server_configs.GetGameServerConfigRequest`):
                 The request object. Request message for
                 GameServerConfigsService.GetGameServerConfig.
             name (:class:`str`):
-                Required. The name of the Game Server Config to
+                Required. The name of the game server config to
                 retrieve. Uses the form:
 
                 ``projects/{project}/locations/{location}/gameServerDeployments/{deployment}/configs/{config}``.
@@ -261,7 +358,7 @@ class GameServerConfigsServiceClient(metaclass=GameServerConfigsServiceClientMet
 
         Returns:
             ~.game_server_configs.GameServerConfig:
-                A Game Server Config resource.
+                A game server config resource.
         """
         # Create or coerce a protobuf request object.
         # Sanity check: If we got a request object, we should *not* have
@@ -276,6 +373,7 @@ class GameServerConfigsServiceClient(metaclass=GameServerConfigsServiceClientMet
 
         # If we have keyword arguments corresponding to fields on the
         # request, apply these.
+
         if name is not None:
             request.name = name
 
@@ -294,7 +392,7 @@ class GameServerConfigsServiceClient(metaclass=GameServerConfigsServiceClientMet
         )
 
         # Send the request.
-        response = rpc(request, retry=retry, timeout=timeout, metadata=metadata)
+        response = rpc(request, retry=retry, timeout=timeout, metadata=metadata,)
 
         # Done; return the response.
         return response
@@ -309,10 +407,10 @@ class GameServerConfigsServiceClient(metaclass=GameServerConfigsServiceClientMet
         timeout: float = None,
         metadata: Sequence[Tuple[str, str]] = (),
     ) -> operation.Operation:
-        r"""Creates a new Game Server Config in a given project,
-        Location, and Game Server Deployment. Game Server
-        Configs are immutable, and are not applied until
-        referenced in the Game Server Deployment Rollout
+        r"""Creates a new game server config in a given project,
+        location, and game server deployment. Game server
+        configs are immutable, and are not applied until
+        referenced in the game server deployment rollout
         resource.
 
         Args:
@@ -327,7 +425,7 @@ class GameServerConfigsServiceClient(metaclass=GameServerConfigsServiceClientMet
                 on the ``request`` instance; if ``request`` is provided, this
                 should not be set.
             game_server_config (:class:`~.game_server_configs.GameServerConfig`):
-                Required. The Game Server Config
+                Required. The game server config
                 resource to be created.
                 This corresponds to the ``game_server_config`` field
                 on the ``request`` instance; if ``request`` is provided, this
@@ -345,7 +443,7 @@ class GameServerConfigsServiceClient(metaclass=GameServerConfigsServiceClientMet
 
                 The result type for the operation will be
                 :class:``~.game_server_configs.GameServerConfig``: A
-                Game Server Config resource.
+                game server config resource.
 
         """
         # Create or coerce a protobuf request object.
@@ -361,10 +459,9 @@ class GameServerConfigsServiceClient(metaclass=GameServerConfigsServiceClientMet
 
         # If we have keyword arguments corresponding to fields on the
         # request, apply these.
+
         if parent is not None:
             request.parent = parent
-        # If we have keyword arguments corresponding to fields on the
-        # request, apply these.
         if game_server_config is not None:
             request.game_server_config = game_server_config
 
@@ -376,8 +473,14 @@ class GameServerConfigsServiceClient(metaclass=GameServerConfigsServiceClientMet
             client_info=_client_info,
         )
 
+        # Certain fields should be provided within the metadata header;
+        # add these here.
+        metadata = tuple(metadata) + (
+            gapic_v1.routing_header.to_grpc_metadata((("parent", request.parent),)),
+        )
+
         # Send the request.
-        response = rpc(request, retry=retry, timeout=timeout, metadata=metadata)
+        response = rpc(request, retry=retry, timeout=timeout, metadata=metadata,)
 
         # Wrap the response in an operation future.
         response = operation.from_gapic(
@@ -399,16 +502,16 @@ class GameServerConfigsServiceClient(metaclass=GameServerConfigsServiceClientMet
         timeout: float = None,
         metadata: Sequence[Tuple[str, str]] = (),
     ) -> operation.Operation:
-        r"""Deletes a single Game Server Config. The deletion
-        will fail if the Game Server Config is referenced in a
-        Game Server Deployment Rollout.
+        r"""Deletes a single game server config. The deletion
+        will fail if the game server config is referenced in a
+        game server deployment rollout.
 
         Args:
             request (:class:`~.game_server_configs.DeleteGameServerConfigRequest`):
                 The request object. Request message for
                 GameServerConfigsService.DeleteGameServerConfig.
             name (:class:`str`):
-                Required. The name of the Game Server Config to delete.
+                Required. The name of the game server config to delete.
                 Uses the form:
 
                 ``projects/{project}/locations/{location}/gameServerDeployments/{deployment}/configs/{config}``.
@@ -427,8 +530,20 @@ class GameServerConfigsServiceClient(metaclass=GameServerConfigsServiceClientMet
                 An object representing a long-running operation.
 
                 The result type for the operation will be
-                :class:``~.game_server_configs.GameServerConfig``: A
-                Game Server Config resource.
+                :class:``~.empty.Empty``: A generic empty message that
+                you can re-use to avoid defining duplicated empty
+                messages in your APIs. A typical example is to use it as
+                the request or the response type of an API method. For
+                instance:
+
+                ::
+
+                    service Foo {
+                      rpc Bar(google.protobuf.Empty) returns (google.protobuf.Empty);
+                    }
+
+                The JSON representation for ``Empty`` is empty JSON
+                object ``{}``.
 
         """
         # Create or coerce a protobuf request object.
@@ -444,6 +559,7 @@ class GameServerConfigsServiceClient(metaclass=GameServerConfigsServiceClientMet
 
         # If we have keyword arguments corresponding to fields on the
         # request, apply these.
+
         if name is not None:
             request.name = name
 
@@ -455,14 +571,20 @@ class GameServerConfigsServiceClient(metaclass=GameServerConfigsServiceClientMet
             client_info=_client_info,
         )
 
+        # Certain fields should be provided within the metadata header;
+        # add these here.
+        metadata = tuple(metadata) + (
+            gapic_v1.routing_header.to_grpc_metadata((("name", request.name),)),
+        )
+
         # Send the request.
-        response = rpc(request, retry=retry, timeout=timeout, metadata=metadata)
+        response = rpc(request, retry=retry, timeout=timeout, metadata=metadata,)
 
         # Wrap the response in an operation future.
         response = operation.from_gapic(
             response,
             self._transport.operations_client,
-            game_server_configs.GameServerConfig,
+            empty.Empty,
             metadata_type=common.OperationMetadata,
         )
 
@@ -473,8 +595,8 @@ class GameServerConfigsServiceClient(metaclass=GameServerConfigsServiceClientMet
 try:
     _client_info = gapic_v1.client_info.ClientInfo(
         gapic_version=pkg_resources.get_distribution(
-            "google-cloud-game-servers"
-        ).version
+            "google-cloud-game-servers",
+        ).version,
     )
 except pkg_resources.DistributionNotFound:
     _client_info = gapic_v1.client_info.ClientInfo()
