@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 
-# Copyright (C) 2019  Google LLC
+# Copyright 2020 Google LLC
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -20,6 +20,7 @@ from typing import Dict, Type
 
 from .base import GameServerDeploymentsServiceTransport
 from .grpc import GameServerDeploymentsServiceGrpcTransport
+from .grpc_asyncio import GameServerDeploymentsServiceGrpcAsyncIOTransport
 
 
 # Compile a registry of transports.
@@ -27,9 +28,11 @@ _transport_registry = (
     OrderedDict()
 )  # type: Dict[str, Type[GameServerDeploymentsServiceTransport]]
 _transport_registry["grpc"] = GameServerDeploymentsServiceGrpcTransport
+_transport_registry["grpc_asyncio"] = GameServerDeploymentsServiceGrpcAsyncIOTransport
 
 
 __all__ = (
     "GameServerDeploymentsServiceTransport",
     "GameServerDeploymentsServiceGrpcTransport",
+    "GameServerDeploymentsServiceGrpcAsyncIOTransport",
 )
