@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 
-# Copyright (C) 2019  Google LLC
+# Copyright 2020 Google LLC
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -15,11 +15,14 @@
 # limitations under the License.
 #
 
-from typing import Callable, Dict
+from typing import Callable, Dict, Optional, Sequence, Tuple
 
 from google.api_core import grpc_helpers  # type: ignore
 from google.api_core import operations_v1  # type: ignore
+from google import auth  # type: ignore
 from google.auth import credentials  # type: ignore
+from google.auth.transport.grpc import SslCredentials  # type: ignore
+
 
 import grpc  # type: ignore
 
@@ -32,7 +35,7 @@ from .base import GameServerDeploymentsServiceTransport
 class GameServerDeploymentsServiceGrpcTransport(GameServerDeploymentsServiceTransport):
     """gRPC backend transport for GameServerDeploymentsService.
 
-    The Game Server Deployment is used to control the deployment
+    The game server deployment is used to control the deployment
     of Agones fleets.
 
     This class defines the same methods as the primary client, so the
@@ -43,12 +46,18 @@ class GameServerDeploymentsServiceGrpcTransport(GameServerDeploymentsServiceTran
     top of HTTP/2); the ``grpcio`` package must be installed.
     """
 
+    _stubs: Dict[str, Callable]
+
     def __init__(
         self,
         *,
         host: str = "gameservices.googleapis.com",
         credentials: credentials.Credentials = None,
-        channel: grpc.Channel = None
+        credentials_file: str = None,
+        scopes: Sequence[str] = None,
+        channel: grpc.Channel = None,
+        api_mtls_endpoint: str = None,
+        client_cert_source: Callable[[], Tuple[bytes, bytes]] = None
     ) -> None:
         """Instantiate the transport.
 
@@ -60,27 +69,81 @@ class GameServerDeploymentsServiceGrpcTransport(GameServerDeploymentsServiceTran
                 are specified, the client will attempt to ascertain the
                 credentials from the environment.
                 This argument is ignored if ``channel`` is provided.
+            credentials_file (Optional[str]): A file with credentials that can
+                be loaded with :func:`google.auth.load_credentials_from_file`.
+                This argument is ignored if ``channel`` is provided.
+            scopes (Optional(Sequence[str])): A list of scopes. This argument is
+                ignored if ``channel`` is provided.
             channel (Optional[grpc.Channel]): A ``Channel`` instance through
                 which to make calls.
+            api_mtls_endpoint (Optional[str]): The mutual TLS endpoint. If
+                provided, it overrides the ``host`` argument and tries to create
+                a mutual TLS channel with client SSL credentials from
+                ``client_cert_source`` or applicatin default SSL credentials.
+            client_cert_source (Optional[Callable[[], Tuple[bytes, bytes]]]): A
+                callback to provide client SSL certificate bytes and private key
+                bytes, both in PEM format. It is ignored if ``api_mtls_endpoint``
+                is None.
+
+        Raises:
+          google.auth.exceptions.MutualTLSChannelError: If mutual TLS transport
+              creation failed for any reason.
+          google.api_core.exceptions.DuplicateCredentialArgs: If both ``credentials``
+              and ``credentials_file`` are passed.
         """
-        # Sanity check: Ensure that channel and credentials are not both
-        # provided.
         if channel:
+            # Sanity check: Ensure that channel and credentials are not both
+            # provided.
             credentials = False
 
-        # Run the base constructor.
-        super().__init__(host=host, credentials=credentials)
-        self._stubs = {}  # type: Dict[str, Callable]
-
-        # If a channel was explicitly provided, set it.
-        if channel:
+            # If a channel was explicitly provided, set it.
             self._grpc_channel = channel
+        elif api_mtls_endpoint:
+            host = (
+                api_mtls_endpoint
+                if ":" in api_mtls_endpoint
+                else api_mtls_endpoint + ":443"
+            )
+
+            if credentials is None:
+                credentials, _ = auth.default(scopes=self.AUTH_SCOPES)
+
+            # Create SSL credentials with client_cert_source or application
+            # default SSL credentials.
+            if client_cert_source:
+                cert, key = client_cert_source()
+                ssl_credentials = grpc.ssl_channel_credentials(
+                    certificate_chain=cert, private_key=key
+                )
+            else:
+                ssl_credentials = SslCredentials().ssl_credentials
+
+            # create a new channel. The provided one is ignored.
+            self._grpc_channel = type(self).create_channel(
+                host,
+                credentials=credentials,
+                credentials_file=credentials_file,
+                ssl_credentials=ssl_credentials,
+                scopes=scopes or self.AUTH_SCOPES,
+            )
+
+        # Run the base constructor.
+        super().__init__(
+            host=host,
+            credentials=credentials,
+            credentials_file=credentials_file,
+            scopes=scopes or self.AUTH_SCOPES,
+        )
+
+        self._stubs = {}  # type: Dict[str, Callable]
 
     @classmethod
     def create_channel(
         cls,
         host: str = "gameservices.googleapis.com",
         credentials: credentials.Credentials = None,
+        credentials_file: str = None,
+        scopes: Optional[Sequence[str]] = None,
         **kwargs
     ) -> grpc.Channel:
         """Create and return a gRPC channel object.
@@ -91,13 +154,28 @@ class GameServerDeploymentsServiceGrpcTransport(GameServerDeploymentsServiceTran
                 credentials identify this application to the service. If
                 none are specified, the client will attempt to ascertain
                 the credentials from the environment.
+            credentials_file (Optional[str]): A file with credentials that can
+                be loaded with :func:`google.auth.load_credentials_from_file`.
+                This argument is mutually exclusive with credentials.
+            scopes (Optional[Sequence[str]]): A optional list of scopes needed for this
+                service. These are only used when credentials are not specified and
+                are passed to :func:`google.auth.default`.
             kwargs (Optional[dict]): Keyword arguments, which are passed to the
                 channel creation.
         Returns:
             grpc.Channel: A gRPC channel object.
+
+        Raises:
+            google.api_core.exceptions.DuplicateCredentialArgs: If both ``credentials``
+              and ``credentials_file`` are passed.
         """
+        scopes = scopes or cls.AUTH_SCOPES
         return grpc_helpers.create_channel(
-            host, credentials=credentials, scopes=cls.AUTH_SCOPES, **kwargs
+            host,
+            credentials=credentials,
+            credentials_file=credentials_file,
+            scopes=scopes,
+            **kwargs
         )
 
     @property
@@ -111,7 +189,7 @@ class GameServerDeploymentsServiceGrpcTransport(GameServerDeploymentsServiceTran
         # have one.
         if not hasattr(self, "_grpc_channel"):
             self._grpc_channel = self.create_channel(
-                self._host, credentials=self._credentials
+                self._host, credentials=self._credentials,
             )
 
         # Return the channel from cache.
@@ -135,15 +213,15 @@ class GameServerDeploymentsServiceGrpcTransport(GameServerDeploymentsServiceTran
 
     @property
     def list_game_server_deployments(
-        self
+        self,
     ) -> Callable[
         [game_server_deployments.ListGameServerDeploymentsRequest],
         game_server_deployments.ListGameServerDeploymentsResponse,
     ]:
         r"""Return a callable for the list game server deployments method over gRPC.
 
-        Lists Game Server Deployments in a given project and
-        Location.
+        Lists game server deployments in a given project and
+        location.
 
         Returns:
             Callable[[~.ListGameServerDeploymentsRequest],
@@ -165,14 +243,14 @@ class GameServerDeploymentsServiceGrpcTransport(GameServerDeploymentsServiceTran
 
     @property
     def get_game_server_deployment(
-        self
+        self,
     ) -> Callable[
         [game_server_deployments.GetGameServerDeploymentRequest],
         game_server_deployments.GameServerDeployment,
     ]:
         r"""Return a callable for the get game server deployment method over gRPC.
 
-        Gets details of a single Game Server Deployment.
+        Gets details of a single game server deployment.
 
         Returns:
             Callable[[~.GetGameServerDeploymentRequest],
@@ -194,15 +272,15 @@ class GameServerDeploymentsServiceGrpcTransport(GameServerDeploymentsServiceTran
 
     @property
     def create_game_server_deployment(
-        self
+        self,
     ) -> Callable[
         [game_server_deployments.CreateGameServerDeploymentRequest],
         operations.Operation,
     ]:
         r"""Return a callable for the create game server deployment method over gRPC.
 
-        Creates a new Game Server Deployment in a given
-        project and Location.
+        Creates a new game server deployment in a given
+        project and location.
 
         Returns:
             Callable[[~.CreateGameServerDeploymentRequest],
@@ -226,14 +304,14 @@ class GameServerDeploymentsServiceGrpcTransport(GameServerDeploymentsServiceTran
 
     @property
     def delete_game_server_deployment(
-        self
+        self,
     ) -> Callable[
         [game_server_deployments.DeleteGameServerDeploymentRequest],
         operations.Operation,
     ]:
         r"""Return a callable for the delete game server deployment method over gRPC.
 
-        Deletes a single Game Server Deployment.
+        Deletes a single game server deployment.
 
         Returns:
             Callable[[~.DeleteGameServerDeploymentRequest],
@@ -257,14 +335,14 @@ class GameServerDeploymentsServiceGrpcTransport(GameServerDeploymentsServiceTran
 
     @property
     def update_game_server_deployment(
-        self
+        self,
     ) -> Callable[
         [game_server_deployments.UpdateGameServerDeploymentRequest],
         operations.Operation,
     ]:
         r"""Return a callable for the update game server deployment method over gRPC.
 
-        Patches a Game Server Deployment.
+        Patches a game server deployment.
 
         Returns:
             Callable[[~.UpdateGameServerDeploymentRequest],
@@ -288,7 +366,7 @@ class GameServerDeploymentsServiceGrpcTransport(GameServerDeploymentsServiceTran
 
     @property
     def get_game_server_deployment_rollout(
-        self
+        self,
     ) -> Callable[
         [game_server_deployments.GetGameServerDeploymentRolloutRequest],
         game_server_deployments.GameServerDeploymentRollout,
@@ -296,7 +374,7 @@ class GameServerDeploymentsServiceGrpcTransport(GameServerDeploymentsServiceTran
         r"""Return a callable for the get game server deployment
         rollout method over gRPC.
 
-        Gets details a single Game Server Deployment Rollout.
+        Gets details a single game server deployment rollout.
 
         Returns:
             Callable[[~.GetGameServerDeploymentRolloutRequest],
@@ -320,7 +398,7 @@ class GameServerDeploymentsServiceGrpcTransport(GameServerDeploymentsServiceTran
 
     @property
     def update_game_server_deployment_rollout(
-        self
+        self,
     ) -> Callable[
         [game_server_deployments.UpdateGameServerDeploymentRolloutRequest],
         operations.Operation,
@@ -328,7 +406,7 @@ class GameServerDeploymentsServiceGrpcTransport(GameServerDeploymentsServiceTran
         r"""Return a callable for the update game server deployment
         rollout method over gRPC.
 
-        Patches a single Game Server Deployment Rollout. The method will
+        Patches a single game server deployment rollout. The method will
         not return an error if the update does not affect any existing
         realms. For example - if the default_game_server_config is
         changed but all existing realms use the override, that is valid.
@@ -358,7 +436,7 @@ class GameServerDeploymentsServiceGrpcTransport(GameServerDeploymentsServiceTran
 
     @property
     def preview_game_server_deployment_rollout(
-        self
+        self,
     ) -> Callable[
         [game_server_deployments.PreviewGameServerDeploymentRolloutRequest],
         game_server_deployments.PreviewGameServerDeploymentRolloutResponse,
@@ -366,8 +444,8 @@ class GameServerDeploymentsServiceGrpcTransport(GameServerDeploymentsServiceTran
         r"""Return a callable for the preview game server deployment
         rollout method over gRPC.
 
-        Previews the Game Server Deployment Rollout. This API
-        does not mutate the Rollout resource.
+        Previews the game server deployment rollout. This API
+        does not mutate the rollout resource.
 
         Returns:
             Callable[[~.PreviewGameServerDeploymentRolloutRequest],
@@ -391,7 +469,7 @@ class GameServerDeploymentsServiceGrpcTransport(GameServerDeploymentsServiceTran
 
     @property
     def fetch_deployment_state(
-        self
+        self,
     ) -> Callable[
         [game_server_deployments.FetchDeploymentStateRequest],
         game_server_deployments.FetchDeploymentStateResponse,
@@ -399,9 +477,9 @@ class GameServerDeploymentsServiceGrpcTransport(GameServerDeploymentsServiceTran
         r"""Return a callable for the fetch deployment state method over gRPC.
 
         Retrieves information about the current state of the
-        Game Server Ddeployment. Gathers all the Agones fleets
+        game server deployment. Gathers all the Agones fleets
         and Agones autoscalers, including fleets running an
-        older version of the Game Server Deployment.
+        older version of the game server deployment.
 
         Returns:
             Callable[[~.FetchDeploymentStateRequest],
