@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 
-# Copyright (C) 2019  Google LLC
+# Copyright 2020 Google LLC
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -16,7 +16,9 @@
 #
 
 from collections import OrderedDict
-from typing import Dict, Sequence, Tuple, Type, Union
+import os
+import re
+from typing import Callable, Dict, Sequence, Tuple, Type, Union
 import pkg_resources
 
 import google.api_core.client_options as ClientOptions  # type: ignore
@@ -24,17 +26,22 @@ from google.api_core import exceptions  # type: ignore
 from google.api_core import gapic_v1  # type: ignore
 from google.api_core import retry as retries  # type: ignore
 from google.auth import credentials  # type: ignore
+from google.auth.transport import mtls  # type: ignore
+from google.auth.exceptions import MutualTLSChannelError  # type: ignore
 from google.oauth2 import service_account  # type: ignore
 
 from google.api_core import operation
+from google.api_core import operation_async
 from google.cloud.gaming_v1.services.realms_service import pagers
 from google.cloud.gaming_v1.types import common
 from google.cloud.gaming_v1.types import realms
+from google.protobuf import empty_pb2 as empty  # type: ignore
 from google.protobuf import field_mask_pb2 as field_mask  # type: ignore
 from google.protobuf import timestamp_pb2 as timestamp  # type: ignore
 
 from .transports.base import RealmsServiceTransport
 from .transports.grpc import RealmsServiceGrpcTransport
+from .transports.grpc_asyncio import RealmsServiceGrpcAsyncIOTransport
 
 
 class RealmsServiceClientMeta(type):
@@ -47,8 +54,9 @@ class RealmsServiceClientMeta(type):
 
     _transport_registry = OrderedDict()  # type: Dict[str, Type[RealmsServiceTransport]]
     _transport_registry["grpc"] = RealmsServiceGrpcTransport
+    _transport_registry["grpc_asyncio"] = RealmsServiceGrpcAsyncIOTransport
 
-    def get_transport_class(cls, label: str = None) -> Type[RealmsServiceTransport]:
+    def get_transport_class(cls, label: str = None,) -> Type[RealmsServiceTransport]:
         """Return an appropriate transport class.
 
         Args:
@@ -68,12 +76,42 @@ class RealmsServiceClientMeta(type):
 
 
 class RealmsServiceClient(metaclass=RealmsServiceClientMeta):
-    """A Realm is a grouping of Game Server Clusters that are
+    """A realm is a grouping of game server clusters that are
     considered interchangeable.
     """
 
-    DEFAULT_OPTIONS = ClientOptions.ClientOptions(
-        api_endpoint="gameservices.googleapis.com"
+    @staticmethod
+    def _get_default_mtls_endpoint(api_endpoint):
+        """Convert api endpoint to mTLS endpoint.
+        Convert "*.sandbox.googleapis.com" and "*.googleapis.com" to
+        "*.mtls.sandbox.googleapis.com" and "*.mtls.googleapis.com" respectively.
+        Args:
+            api_endpoint (Optional[str]): the api endpoint to convert.
+        Returns:
+            str: converted mTLS api endpoint.
+        """
+        if not api_endpoint:
+            return api_endpoint
+
+        mtls_endpoint_re = re.compile(
+            r"(?P<name>[^.]+)(?P<mtls>\.mtls)?(?P<sandbox>\.sandbox)?(?P<googledomain>\.googleapis\.com)?"
+        )
+
+        m = mtls_endpoint_re.match(api_endpoint)
+        name, mtls, sandbox, googledomain = m.groups()
+        if mtls or not googledomain:
+            return api_endpoint
+
+        if sandbox:
+            return api_endpoint.replace(
+                "sandbox.googleapis.com", "mtls.sandbox.googleapis.com"
+            )
+
+        return api_endpoint.replace(".googleapis.com", ".mtls.googleapis.com")
+
+    DEFAULT_ENDPOINT = "gameservices.googleapis.com"
+    DEFAULT_MTLS_ENDPOINT = _get_default_mtls_endpoint.__func__(  # type: ignore
+        DEFAULT_ENDPOINT
     )
 
     @classmethod
@@ -97,18 +135,27 @@ class RealmsServiceClient(metaclass=RealmsServiceClientMeta):
     from_service_account_json = from_service_account_file
 
     @staticmethod
-    def realm_path(project: str, location: str, realm: str) -> str:
+    def realm_path(project: str, location: str, realm: str,) -> str:
         """Return a fully-qualified realm string."""
         return "projects/{project}/locations/{location}/realms/{realm}".format(
-            project=project, location=location, realm=realm
+            project=project, location=location, realm=realm,
         )
 
+    @staticmethod
+    def parse_realm_path(path: str) -> Dict[str, str]:
+        """Parse a realm path into its component segments."""
+        m = re.match(
+            r"^projects/(?P<project>.+?)/locations/(?P<location>.+?)/realms/(?P<realm>.+?)$",
+            path,
+        )
+        return m.groupdict() if m else {}
+
     def __init__(
         self,
         *,
         credentials: credentials.Credentials = None,
         transport: Union[str, RealmsServiceTransport] = None,
-        client_options: ClientOptions = DEFAULT_OPTIONS,
+        client_options: ClientOptions = None,
     ) -> None:
         """Instantiate the realms service client.
 
@@ -121,26 +168,75 @@ class RealmsServiceClient(metaclass=RealmsServiceClientMeta):
             transport (Union[str, ~.RealmsServiceTransport]): The
                 transport to use. If set to None, a transport is chosen
                 automatically.
-            client_options (ClientOptions): Custom options for the client.
+            client_options (ClientOptions): Custom options for the client. It
+                won't take effect if a ``transport`` instance is provided.
+                (1) The ``api_endpoint`` property can be used to override the
+                default endpoint provided by the client. GOOGLE_API_USE_MTLS
+                environment variable can also be used to override the endpoint:
+                "always" (always use the default mTLS endpoint), "never" (always
+                use the default regular endpoint, this is the default value for
+                the environment variable) and "auto" (auto switch to the default
+                mTLS endpoint if client SSL credentials is present). However,
+                the ``api_endpoint`` property takes precedence if provided.
+                (2) The ``client_cert_source`` property is used to provide client
+                SSL credentials for mutual TLS transport. If not provided, the
+                default SSL credentials will be used if present.
+
+        Raises:
+            google.auth.exceptions.MutualTLSChannelError: If mutual TLS transport
+                creation failed for any reason.
         """
         if isinstance(client_options, dict):
             client_options = ClientOptions.from_dict(client_options)
+        if client_options is None:
+            client_options = ClientOptions.ClientOptions()
+
+        if client_options.api_endpoint is None:
+            use_mtls_env = os.getenv("GOOGLE_API_USE_MTLS", "never")
+            if use_mtls_env == "never":
+                client_options.api_endpoint = self.DEFAULT_ENDPOINT
+            elif use_mtls_env == "always":
+                client_options.api_endpoint = self.DEFAULT_MTLS_ENDPOINT
+            elif use_mtls_env == "auto":
+                has_client_cert_source = (
+                    client_options.client_cert_source is not None
+                    or mtls.has_default_client_cert_source()
+                )
+                client_options.api_endpoint = (
+                    self.DEFAULT_MTLS_ENDPOINT
+                    if has_client_cert_source
+                    else self.DEFAULT_ENDPOINT
+                )
+            else:
+                raise MutualTLSChannelError(
+                    "Unsupported GOOGLE_API_USE_MTLS value. Accepted values: never, auto, always"
+                )
 
         # Save or instantiate the transport.
         # Ordinarily, we provide the transport, but allowing a custom transport
         # instance provides an extensibility point for unusual situations.
         if isinstance(transport, RealmsServiceTransport):
-            if credentials:
+            # transport is a RealmsServiceTransport instance.
+            if credentials or client_options.credentials_file:
                 raise ValueError(
                     "When providing a transport instance, "
                     "provide its credentials directly."
                 )
+            if client_options.scopes:
+                raise ValueError(
+                    "When providing a transport instance, "
+                    "provide its scopes directly."
+                )
             self._transport = transport
         else:
             Transport = type(self).get_transport_class(transport)
             self._transport = Transport(
                 credentials=credentials,
-                host=client_options.api_endpoint or "gameservices.googleapis.com",
+                credentials_file=client_options.credentials_file,
+                host=client_options.api_endpoint,
+                scopes=client_options.scopes,
+                api_mtls_endpoint=client_options.api_endpoint,
+                client_cert_source=client_options.client_cert_source,
             )
 
     def list_realms(
@@ -152,7 +248,7 @@ class RealmsServiceClient(metaclass=RealmsServiceClientMeta):
         timeout: float = None,
         metadata: Sequence[Tuple[str, str]] = (),
     ) -> pagers.ListRealmsPager:
-        r"""Lists Realms in a given project and Location.
+        r"""Lists realms in a given project and location.
 
         Args:
             request (:class:`~.realms.ListRealmsRequest`):
@@ -193,13 +289,14 @@ class RealmsServiceClient(metaclass=RealmsServiceClientMeta):
 
         # If we have keyword arguments corresponding to fields on the
         # request, apply these.
+
         if parent is not None:
             request.parent = parent
 
         # Wrap the RPC method; this adds retry and timeout information,
         # and friendly error handling.
         rpc = gapic_v1.method.wrap_method(
-            self._transport.list_realms, default_timeout=None, client_info=_client_info
+            self._transport.list_realms, default_timeout=None, client_info=_client_info,
         )
 
         # Certain fields should be provided within the metadata header;
@@ -209,12 +306,12 @@ class RealmsServiceClient(metaclass=RealmsServiceClientMeta):
         )
 
         # Send the request.
-        response = rpc(request, retry=retry, timeout=timeout, metadata=metadata)
+        response = rpc(request, retry=retry, timeout=timeout, metadata=metadata,)
 
         # This method is paged; wrap the response in a pager, which provides
         # an `__iter__` convenience method.
         response = pagers.ListRealmsPager(
-            method=rpc, request=request, response=response
+            method=rpc, request=request, response=response, metadata=metadata,
         )
 
         # Done; return the response.
@@ -229,14 +326,14 @@ class RealmsServiceClient(metaclass=RealmsServiceClientMeta):
         timeout: float = None,
         metadata: Sequence[Tuple[str, str]] = (),
     ) -> realms.Realm:
-        r"""Gets details of a single Realm.
+        r"""Gets details of a single realm.
 
         Args:
             request (:class:`~.realms.GetRealmRequest`):
                 The request object. Request message for
                 RealmsService.GetRealm.
             name (:class:`str`):
-                Required. The name of the Realm to retrieve. Uses the
+                Required. The name of the realm to retrieve. Uses the
                 form:
                 ``projects/{project}/locations/{location}/realms/{realm}``.
                 This corresponds to the ``name`` field
@@ -251,7 +348,7 @@ class RealmsServiceClient(metaclass=RealmsServiceClientMeta):
 
         Returns:
             ~.realms.Realm:
-                A Realm resource.
+                A realm resource.
         """
         # Create or coerce a protobuf request object.
         # Sanity check: If we got a request object, we should *not* have
@@ -266,13 +363,14 @@ class RealmsServiceClient(metaclass=RealmsServiceClientMeta):
 
         # If we have keyword arguments corresponding to fields on the
         # request, apply these.
+
         if name is not None:
             request.name = name
 
         # Wrap the RPC method; this adds retry and timeout information,
         # and friendly error handling.
         rpc = gapic_v1.method.wrap_method(
-            self._transport.get_realm, default_timeout=None, client_info=_client_info
+            self._transport.get_realm, default_timeout=None, client_info=_client_info,
         )
 
         # Certain fields should be provided within the metadata header;
@@ -282,7 +380,7 @@ class RealmsServiceClient(metaclass=RealmsServiceClientMeta):
         )
 
         # Send the request.
-        response = rpc(request, retry=retry, timeout=timeout, metadata=metadata)
+        response = rpc(request, retry=retry, timeout=timeout, metadata=metadata,)
 
         # Done; return the response.
         return response
@@ -298,7 +396,7 @@ class RealmsServiceClient(metaclass=RealmsServiceClientMeta):
         timeout: float = None,
         metadata: Sequence[Tuple[str, str]] = (),
     ) -> operation.Operation:
-        r"""Creates a new Realm in a given project and Location.
+        r"""Creates a new realm in a given project and location.
 
         Args:
             request (:class:`~.realms.CreateRealmRequest`):
@@ -311,13 +409,13 @@ class RealmsServiceClient(metaclass=RealmsServiceClientMeta):
                 on the ``request`` instance; if ``request`` is provided, this
                 should not be set.
             realm (:class:`~.realms.Realm`):
-                Required. The Realm resource to be
+                Required. The realm resource to be
                 created.
                 This corresponds to the ``realm`` field
                 on the ``request`` instance; if ``request`` is provided, this
                 should not be set.
             realm_id (:class:`str`):
-                Required. The ID of the Realm
+                Required. The ID of the realm
                 resource to be created.
                 This corresponds to the ``realm_id`` field
                 on the ``request`` instance; if ``request`` is provided, this
@@ -334,7 +432,7 @@ class RealmsServiceClient(metaclass=RealmsServiceClientMeta):
                 An object representing a long-running operation.
 
                 The result type for the operation will be
-                :class:``~.realms.Realm``: A Realm resource.
+                :class:``~.realms.Realm``: A realm resource.
 
         """
         # Create or coerce a protobuf request object.
@@ -350,25 +448,30 @@ class RealmsServiceClient(metaclass=RealmsServiceClientMeta):
 
         # If we have keyword arguments corresponding to fields on the
         # request, apply these.
+
         if parent is not None:
             request.parent = parent
-        # If we have keyword arguments corresponding to fields on the
-        # request, apply these.
         if realm is not None:
             request.realm = realm
-        # If we have keyword arguments corresponding to fields on the
-        # request, apply these.
         if realm_id is not None:
             request.realm_id = realm_id
 
         # Wrap the RPC method; this adds retry and timeout information,
         # and friendly error handling.
         rpc = gapic_v1.method.wrap_method(
-            self._transport.create_realm, default_timeout=None, client_info=_client_info
+            self._transport.create_realm,
+            default_timeout=None,
+            client_info=_client_info,
+        )
+
+        # Certain fields should be provided within the metadata header;
+        # add these here.
+        metadata = tuple(metadata) + (
+            gapic_v1.routing_header.to_grpc_metadata((("parent", request.parent),)),
         )
 
         # Send the request.
-        response = rpc(request, retry=retry, timeout=timeout, metadata=metadata)
+        response = rpc(request, retry=retry, timeout=timeout, metadata=metadata,)
 
         # Wrap the response in an operation future.
         response = operation.from_gapic(
@@ -390,14 +493,14 @@ class RealmsServiceClient(metaclass=RealmsServiceClientMeta):
         timeout: float = None,
         metadata: Sequence[Tuple[str, str]] = (),
     ) -> operation.Operation:
-        r"""Deletes a single Realm.
+        r"""Deletes a single realm.
 
         Args:
             request (:class:`~.realms.DeleteRealmRequest`):
                 The request object. Request message for
                 RealmsService.DeleteRealm.
             name (:class:`str`):
-                Required. The name of the Realm to delete. Uses the
+                Required. The name of the realm to delete. Uses the
                 form:
                 ``projects/{project}/locations/{location}/realms/{realm}``.
                 This corresponds to the ``name`` field
@@ -415,7 +518,20 @@ class RealmsServiceClient(metaclass=RealmsServiceClientMeta):
                 An object representing a long-running operation.
 
                 The result type for the operation will be
-                :class:``~.realms.Realm``: A Realm resource.
+                :class:``~.empty.Empty``: A generic empty message that
+                you can re-use to avoid defining duplicated empty
+                messages in your APIs. A typical example is to use it as
+                the request or the response type of an API method. For
+                instance:
+
+                ::
+
+                    service Foo {
+                      rpc Bar(google.protobuf.Empty) returns (google.protobuf.Empty);
+                    }
+
+                The JSON representation for ``Empty`` is empty JSON
+                object ``{}``.
 
         """
         # Create or coerce a protobuf request object.
@@ -431,23 +547,32 @@ class RealmsServiceClient(metaclass=RealmsServiceClientMeta):
 
         # If we have keyword arguments corresponding to fields on the
         # request, apply these.
+
         if name is not None:
             request.name = name
 
         # Wrap the RPC method; this adds retry and timeout information,
         # and friendly error handling.
         rpc = gapic_v1.method.wrap_method(
-            self._transport.delete_realm, default_timeout=None, client_info=_client_info
+            self._transport.delete_realm,
+            default_timeout=None,
+            client_info=_client_info,
+        )
+
+        # Certain fields should be provided within the metadata header;
+        # add these here.
+        metadata = tuple(metadata) + (
+            gapic_v1.routing_header.to_grpc_metadata((("name", request.name),)),
         )
 
         # Send the request.
-        response = rpc(request, retry=retry, timeout=timeout, metadata=metadata)
+        response = rpc(request, retry=retry, timeout=timeout, metadata=metadata,)
 
         # Wrap the response in an operation future.
         response = operation.from_gapic(
             response,
             self._transport.operations_client,
-            realms.Realm,
+            empty.Empty,
             metadata_type=common.OperationMetadata,
         )
 
@@ -464,14 +589,14 @@ class RealmsServiceClient(metaclass=RealmsServiceClientMeta):
         timeout: float = None,
         metadata: Sequence[Tuple[str, str]] = (),
     ) -> operation.Operation:
-        r"""Patches a single Realm.
+        r"""Patches a single realm.
 
         Args:
             request (:class:`~.realms.UpdateRealmRequest`):
                 The request object. Request message for
                 RealmsService.UpdateRealm.
             realm (:class:`~.realms.Realm`):
-                Required. The Realm to be updated. Only fields specified
+                Required. The realm to be updated. Only fields specified
                 in update_mask are updated.
                 This corresponds to the ``realm`` field
                 on the ``request`` instance; if ``request`` is provided, this
@@ -497,7 +622,7 @@ class RealmsServiceClient(metaclass=RealmsServiceClientMeta):
                 An object representing a long-running operation.
 
                 The result type for the operation will be
-                :class:``~.realms.Realm``: A Realm resource.
+                :class:``~.realms.Realm``: A realm resource.
 
         """
         # Create or coerce a protobuf request object.
@@ -513,21 +638,30 @@ class RealmsServiceClient(metaclass=RealmsServiceClientMeta):
 
         # If we have keyword arguments corresponding to fields on the
         # request, apply these.
+
         if realm is not None:
             request.realm = realm
-        # If we have keyword arguments corresponding to fields on the
-        # request, apply these.
         if update_mask is not None:
             request.update_mask = update_mask
 
         # Wrap the RPC method; this adds retry and timeout information,
         # and friendly error handling.
         rpc = gapic_v1.method.wrap_method(
-            self._transport.update_realm, default_timeout=None, client_info=_client_info
+            self._transport.update_realm,
+            default_timeout=None,
+            client_info=_client_info,
+        )
+
+        # Certain fields should be provided within the metadata header;
+        # add these here.
+        metadata = tuple(metadata) + (
+            gapic_v1.routing_header.to_grpc_metadata(
+                (("realm.name", request.realm.name),)
+            ),
         )
 
         # Send the request.
-        response = rpc(request, retry=retry, timeout=timeout, metadata=metadata)
+        response = rpc(request, retry=retry, timeout=timeout, metadata=metadata,)
 
         # Wrap the response in an operation future.
         response = operation.from_gapic(
@@ -548,7 +682,7 @@ class RealmsServiceClient(metaclass=RealmsServiceClientMeta):
         timeout: float = None,
         metadata: Sequence[Tuple[str, str]] = (),
     ) -> realms.PreviewRealmUpdateResponse:
-        r"""Previews patches to a single Realm.
+        r"""Previews patches to a single realm.
 
         Args:
             request (:class:`~.realms.PreviewRealmUpdateRequest`):
@@ -579,8 +713,16 @@ class RealmsServiceClient(metaclass=RealmsServiceClientMeta):
             client_info=_client_info,
         )
 
+        # Certain fields should be provided within the metadata header;
+        # add these here.
+        metadata = tuple(metadata) + (
+            gapic_v1.routing_header.to_grpc_metadata(
+                (("realm.name", request.realm.name),)
+            ),
+        )
+
         # Send the request.
-        response = rpc(request, retry=retry, timeout=timeout, metadata=metadata)
+        response = rpc(request, retry=retry, timeout=timeout, metadata=metadata,)
 
         # Done; return the response.
         return response
@@ -589,8 +731,8 @@ class RealmsServiceClient(metaclass=RealmsServiceClientMeta):
 try:
     _client_info = gapic_v1.client_info.ClientInfo(
         gapic_version=pkg_resources.get_distribution(
-            "google-cloud-game-servers"
-        ).version
+            "google-cloud-game-servers",
+        ).version,
     )
 except pkg_resources.DistributionNotFound:
     _client_info = gapic_v1.client_info.ClientInfo()
