@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 
-# Copyright (C) 2019  Google LLC
+# Copyright 2020 Google LLC
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -20,11 +20,17 @@ from typing import Dict, Type
 
 from .base import RealmsServiceTransport
 from .grpc import RealmsServiceGrpcTransport
+from .grpc_asyncio import RealmsServiceGrpcAsyncIOTransport
 
 
 # Compile a registry of transports.
 _transport_registry = OrderedDict()  # type: Dict[str, Type[RealmsServiceTransport]]
 _transport_registry["grpc"] = RealmsServiceGrpcTransport
+_transport_registry["grpc_asyncio"] = RealmsServiceGrpcAsyncIOTransport
 
 
-__all__ = ("RealmsServiceTransport", "RealmsServiceGrpcTransport")
+__all__ = (
+    "RealmsServiceTransport",
+    "RealmsServiceGrpcTransport",
+    "RealmsServiceGrpcAsyncIOTransport",
+)
